@@ -24,13 +24,13 @@ def parseCliArgs(realArgs):
 def runCommand(realArgs):
   options,args = parseCliArgs(realArgs)
   user = User()
-  runningImages = [container["Image"] for container in user.getDockerDaemon().getContainers(onlyRunning=True)]
-  for _,subuser in user.getRegistry().getSubusers().items():
+  runningImages = [container["Image"] for container in user.dockerDaemon.getContainers(onlyRunning=True)]
+  for _,subuser in user.registry.subusers.items():
     try:
-      if subuser.getRunReadyImage().getId() in runningImages:
+      if subuser.getRunReadyImage().id in runningImages:
         if not options.internal:
-          if subuser.getName().startswith("!"):
+          if subuser.name.startswith("!"):
             continue
-        print(subuser.getName())
+        print(subuser.name)
     except (KeyError,subuserlib.classes.subuserSubmodules.run.runtimeCache.NoRuntimeCacheForSubusersWhichDontHaveExistantImagesException):
       pass
