@@ -4,7 +4,7 @@
 import sys
 import optparse
 #internal imports
-from subuserlib.classes.user import User
+from subuserlib.classes.user import LockedUser
 import subuserlib.verify
 import subuserlib.commandLineArguments
 import subuserlib.profile
@@ -21,15 +21,16 @@ This is usefull when migrating from one machine to another.  You can copy your ~
   parser = optparse.OptionParser(usage=usage,description=description,formatter=subuserlib.commandLineArguments.HelpFormatterThatDoesntReformatDescription())
   parser.add_option("--accept",dest="accept",action="store_true",default=False,help="Acceppt permissions without asking.")
   parser.add_option("--prompt",dest="prompt",action="store_true",default=False,help="Prompt before installing new images.")
+  parser.add_option("--no-build",dest="build",action="store_false",default=True,help="Don't build missing images.")
   return parser.parse_args(args=realArgs)
 
 @subuserlib.profile.do_cprofile
 def runCommand(realArgs):
   options,arguments=parseCliArgs(realArgs)
-  user = User()
-  user.getRegistry().commit_message = " ".join(["subuser","repair"]+realArgs)
-  permissionsAccepter = AcceptPermissionsAtCLI(user,alwaysAccept = options.accept)
-  with user.getRegistry().getLock() as LockFileHandle:
-    subusers = user.getRegistry().getSubusers().getSortedList()
-    subuserlib.verify.verify(user,subusers=subusers,permissionsAccepter=permissionsAccepter,prompt=options.prompt)
-    user.getRegistry().commit()
+  lockedUser = LockedUser()
+  with lockedUser as user:
+    user.registry.commit_message = " ".join(["subuser","repair"]+realArgs)
+    permissionsAccepter = AcceptPermissionsAtCLI(user,alwaysAccept = options.accept)
+    subusers = user.registry.subusers.getSortedList()
+    subuserlib.verify.verify(user,subusers=subusers,permissionsAccepter=permissionsAccepter,prompt=options.prompt,build=options.build)
+    user.registry.commit()
