@@ -5,7 +5,7 @@ import sys
 import optparse
 import os
 #internal imports
-from subuserlib.classes.user import User
+from subuserlib.classes.user import LockedUser
 from subuserlib.classes.permissionsAccepters.acceptPermissionsAtCLI import AcceptPermissionsAtCLI
 import subuserlib.commandLineArguments
 import subuserlib.subuser
@@ -13,16 +13,16 @@ import subuserlib.verify
 import subuserlib.profile
 
 def parseCliArgs(sysargs):
-  usage = "usage: subuser subuser [add|remove|add-to-path|remove-from-path|edit-permissions|expose-entrypoints|hide-entrypoints] NAME [IMAGESOURCE]"
+  usage = "usage: subuser subuser [add|remove|add-to-path|remove-from-path|edit-permissions|expose-entrypoints|hide-entrypoints|change-image] NAME [IMAGESOURCE]"
   description = """
 
 Add and remove subusers.  Create shorcuts for launching subusers.
 
 EXAMPLES:
 
-Add a new subuser named foo based on the image foo@default.
+Add a new subuser named foo based on the image image-name@repository.
 
-    $ subuser subuser add foo foo@default
+    $ subuser subuser add foo image-name@repository
 
 Remove the subuser named foo.
 
@@ -60,9 +60,14 @@ To remove the entrypoints from the PATH use
 Edit a subuser's permissions.
 
     $ subuser subuser edit-permissions foo
+
+Change which image the subuser is associated with.
+
+    $ subuser subuser change-image foo image-name@repository
 """
   parser=optparse.OptionParser(usage=usage,description=description,formatter=subuserlib.commandLineArguments.HelpFormatterThatDoesntReformatDescription())
   parser.add_option("--prefix",dest="prefix",default=None,help="When removing subusers, remove all subusers who's names start with prefix.")
+  parser.add_option("--home-dir",dest="homeDir",default=None,help="When adding a subuser set its home dir on the host to a non-default location.")
   parser.add_option("--accept",dest="accept",action="store_true",default=False,help="Accept permissions without asking.")
   parser.add_option("--prompt",dest="prompt",action="store_true",default=False,help="Prompt before installing new images.")
   parser.add_option("--force-internal",dest="forceInternal",action="store_true",default=False,help="Force a subuser who's name starts with ! to be added, despite the fact that ! marks interal subusers and is normally forbidden.")
@@ -79,27 +84,32 @@ def runCommand(sysargs):
   except IndexError:
     print("Wrong number of arguments!")
     parseCliArgs(["--help"])
-  user = User()
-  user.getRegistry().commit_message = " ".join(["subuser","subuser"]+sysargs)
-  permissionsAccepter = AcceptPermissionsAtCLI(user,alwaysAccept = options.accept)
-  if action == "add":
-    if not len(args) == 3:
-      sys.exit("Wrong number of arguments to add.  See `subuser subuser -h`.")
-    subuserName = args[1]
-    imageSourceId = args[2]
-    with user.getRegistry().getLock():
-      subuserlib.subuser.add(user,subuserName,imageSourceId,permissionsAccepter=permissionsAccepter,prompt=options.prompt,forceInternal=options.forceInternal)
-  else:
-    subuserNames = list(set(args[1:]))
-    with user.getRegistry().getLock():
+  lockedUser = LockedUser()
+  with lockedUser as user:
+    user.registry.commit_message = " ".join(["subuser","subuser"]+sysargs)
+    permissionsAccepter = AcceptPermissionsAtCLI(user,alwaysAccept = options.accept)
+    if action == "add" or action == "change-image":
+      if not len(args) == 3:
+        sys.exit("Wrong number of arguments to add.  See `subuser subuser -h`.")
+      subuserName = args[1]
+      imageSourceId = args[2]
+      if action == "add":
+        if options.homeDir is not None:
+          homeDir = os.path.expanduser(options.homeDir)
+        else:
+          homeDir = None
+        subuserlib.subuser.add(user,subuserName,imageSourceId,permissionsAccepter=permissionsAccepter,prompt=options.prompt,forceInternal=options.forceInternal,homeDir=homeDir)
+      elif action == "change-image":
+        subuserlib.subuser.changeImage(user,subuserName,imageSourceId,permissionsAccepter=permissionsAccepter,prompt=options.prompt)
+    else:
+      subuserNames = list(set(args[1:]))
       subusers = []
       if not options.prefix is None:
-        allSubuserNames = user.getRegistry().getSubusers().keys()
+        allSubuserNames = user.registry.subusers.keys()
         subuserNames.extend([subuserName for subuserName in allSubuserNames if subuserName.startswith(options.prefix)])
-
       for subuserName in subuserNames:
         try:
-          subusers.append(user.getRegistry().getSubusers()[subuserName])
+          subusers.append(user.registry.subusers[subuserName])
         except KeyError:
           sys.exit("Subuser "+subuserName+" does not exist. Use --help for help.")
       if subusers == []:
@@ -117,10 +127,10 @@ def runCommand(sysargs):
           command(user,subusers,install)
           sys.exit()
       if action == "edit-permissions":
-        user.getRegistry().logChange("Edit the permissions of:"+ " ".join(subuserNames))
+        user.registry.logChange("Edit the permissions of:"+ " ".join(subuserNames))
         for subuser in subusers:
           subuser.editPermissionsCLI()
         subuserlib.verify.verify(user,subusers=subusers,permissionsAccepter=permissionsAccepter,prompt=options.prompt)
-        user.getRegistry().commit()
+        user.registry.commit()
         sys.exit()
       sys.exit("Action "+args[0]+" does not exist. Try:\n subuser subuser --help")
