@@ -53,16 +53,18 @@ def archiveBuildContext(archive,relativeBuildContextPath,repositoryFileStructure
   contexttarfile = tarfile.open(mode="w",fileobj=archive)
   if relativeBuildContextPath and repositoryFileStructure:
     def addFolder(folder):
-      for filePathRelativeToRepository in repositoryFileStructure.lsFiles(folder):
+      for filename in repositoryFileStructure.lsFiles(folder):
+        filePathRelativeToRepository = os.path.join(folder,filename)
         filePathRelativeToBuildContext = os.path.relpath(filePathRelativeToRepository,relativeBuildContextPath)
         exclude = False
         for excludePattern in excludePatterns:
           if fnmatch.fnmatch(filePathRelativeToBuildContext,excludePattern):
             exclude = True
+            break
         if not exclude:
           addFileFromContents(path=filePathRelativeToBuildContext,contents=repositoryFileStructure.readBinary(filePathRelativeToRepository),mode=repositoryFileStructure.getMode(filePathRelativeToRepository))
-      for folderPathRelativeToRepository in repositoryFileStructure.lsFolders(folder):
-        addFolder(folderPathRelativeToRepository)
+      for subFolder in repositoryFileStructure.lsFolders(folder):
+        addFolder(os.path.join(folder,subFolder))
     addFolder(relativeBuildContextPath)
   # Add the provided Dockerfile if necessary
   if not dockerfile == None:
@@ -83,9 +85,9 @@ def readAndPrintStreamingBuildStatus(user,response):
       if lineDict == {}:
         pass
       elif "stream" in lineDict:
-        user.getRegistry().log(lineDict["stream"])
+        user.registry.log(lineDict["stream"])
       elif "status" in lineDict:
-        user.getRegistry().log(lineDict["status"])
+        user.registry.log(lineDict["status"])
       elif "errorDetail" in lineDict:
         raise exceptions.ImageBuildException("Build error:"+lineDict["errorDetail"]["message"]+"\n"+response.read().decode())
       else:
@@ -93,7 +95,10 @@ def readAndPrintStreamingBuildStatus(user,response):
       jsonSegmentBytes = b''
     except ValueError:
       pass
-  return output.decode("utf-8")
+  output = output.decode("utf-8")
+  if not output.strip().startswith("{"):
+    user.registry.log(output)
+  return output
 
 class DockerDaemon(UserOwnedObject):
   def __init__(self,user):
@@ -126,7 +131,7 @@ class DockerDaemon(UserOwnedObject):
       return []
 
   def getContainer(self,containerId):
-    return Container(self.getUser(),containerId)
+    return Container(self.user,containerId)
 
   def getImageProperties(self,imageTagOrId):
     """
@@ -139,7 +144,7 @@ class DockerDaemon(UserOwnedObject):
     self.getConnection().request("GET","/v1.13/images/"+imageTagOrId+"/json")
     response = self.getConnection().getresponse()
     if not response.status == 200:
-      response.read() # Read the response and discard it to prevent the server from getting locked up: http://stackoverflow.com/questions/3231543/python-httplib-responsenotready
+      response.read() # Read the response and discard it to prevent the server from getting locked up: https://stackoverflow.com/questions/3231543/python-httplib-responsenotready
       return None
     else:
       properties = json.loads(response.read().decode("utf-8"))
@@ -192,22 +197,24 @@ class DockerDaemon(UserOwnedObject):
       if quietClient:
         response.read()
       else:
-        readAndPrintStreamingBuildStatus(self.getUser(), response)
+        readAndPrintStreamingBuildStatus(self.user, response)
       raise exceptions.ImageBuildException("Building image failed.\n"
                      +"status: "+str(response.status)+"\n"
                      +"Reason: "+response.reason+"\n")
     if quietClient:
       output = response.read().decode("utf-8")
     else:
-      output = readAndPrintStreamingBuildStatus(self.getUser(),response)
+      output = readAndPrintStreamingBuildStatus(self.user,response)
     # Now we move to regex code stolen from the official python Docker bindings. This is REALLY UGLY!
     outputLines = output.split("\n")
     search = r'Successfully built ([0-9a-f]+)' #This is REALLY ugly!
-    match = re.search(search, outputLines[-1]) #This is REALLY ugly!
+    match = None
+    for line in reversed(outputLines):
+      match = re.search(search, line) #This is REALLY ugly!
+      if match:
+        break
     if not match:
-      match = re.search(search, outputLines[-2]) #This is REALLY ugly!
-    if not match:
-      raise exceptions.ImageBuildException("Unexpected server response when building image:\n"+output)
+      raise exceptions.ImageBuildException("Unexpected server response when building image.")
     shortId = match.group(1) #This is REALLY ugly!
     return self.getImageProperties(shortId)["Id"]
 
@@ -218,7 +225,7 @@ class DockerDaemon(UserOwnedObject):
     self.getConnection().request("GET","/v1.13/info")
     response = self.getConnection().getresponse()
     if not response.status == 200:
-      response.read() # Read the response and discard it to prevent the server from getting locked up: http://stackoverflow.com/questions/3231543/python-httplib-responsenotready
+      response.read() # Read the response and discard it to prevent the server from getting locked up: https://stackoverflow.com/questions/3231543/python-httplib-responsenotready
       return None
     else:
       return json.loads(response.read().decode("utf-8"))
