@@ -7,6 +7,7 @@ Images in subuser are built from ImageSource objects.
 #external imports
 import os
 import uuid
+from collections import OrderedDict
 #internal imports
 from subuserlib.classes.userOwnedObject import UserOwnedObject
 from subuserlib.classes.describable import Describable
@@ -18,8 +19,8 @@ import subuserlib.print
 
 class ImageSource(UserOwnedObject,Describable):
   def __init__(self,user,repo,name,explicitConfig=None):
-    self.__name = name
-    self.__repo = repo
+    self.name = name
+    self.repo = repo
     self.__permissions = None
     self.__explicitConfig = explicitConfig
     UserOwnedObject.__init__(self,user)
@@ -27,32 +28,32 @@ class ImageSource(UserOwnedObject,Describable):
   def __hash__(self):
     return hash(self.getIdentifier())
 
-  def getName(self):
-    return self.__name
-
   def getIdentifier(self):
     """
     Return a standard human readable identifier for an ImageSource.
     """
-    return self.getName() + "@" + self.getRepository().getDisplayName()
+    return self.name + "@" + self.repo.displayName
+
+  def serializeToDict(self):
+    return OrderedDict(
+        [("name", self.name)
+        ,("repo", self.repo.displayName)
+        ,("identifier",self.getIdentifier())
+        ,("permissions",self.permissions)
+        ,("installed-images",self.installedImagesDict)
+        ,("latest-installed-image",self.getLatestInstalledImage().imageId)])
 
   def getDockerImageTag(self):
-    longTag = "subuser-" + self.getUser().getEndUser().name + "-" + self.getIdentifier()
+    longTag = "subuser-" + self.user.endUser.name + "-" + self.getIdentifier()
     return subuserlib.docker.buildImageTag(longTag,self.getHash())
 
-  def getRepository(self):
-    """
-    Get the repository where this ImageSource resides.
-    """
-    return self.__repo
-
   def getSubusers(self):
     """
      Get a list of subusers that were built from this ImageSource.
     """
     subusers = []
-    for subuser in self.getUser().getRegistry().getSubusers():
-      if subuser.getImageSource()==self:
+    for subuser in self.user.registry.subusers:
+      if subuser.imageSource==self:
         subusers.append(subuser)
     return subusers
 
@@ -62,17 +63,17 @@ class ImageSource(UserOwnedObject,Describable):
     imageDir = os.path.join(self.getRelativeSourceDir(),"image")
     # If the image dir does not exist,
     # Look for the old, deprecated, docker-image dir
-    if not self.getRepository().getFileStructure().exists(imageDir):
+    if not self.repo.fileStructure.exists(imageDir):
       imageDir = os.path.join(self.getRelativeSourceDir(),"docker-image")
-      if not self.getRepository().getFileStructure().exists(imageDir):
+      if not self.repo.fileStructure.exists(imageDir):
         raise exceptions.ImageBuildException("Image source "+self.getIdentifier()+ " does not have an image dir with sources from which to build.")
     return imageDir
 
   def getSourceDir(self):
-    return os.path.join(self.getRepository().getImageSourcesDir(),self.getName())
+    return os.path.join(self.repo.imageSourcesDir,self.name)
 
   def getRelativeSourceDir(self):
-    return os.path.join(self.getRepository().getRelativeImageSourcesDir(),self.getName())
+    return os.path.join(self.repo.relativeImageSourcesDir,self.name)
 
   def getLatestInstalledImage(self):
     """
@@ -81,49 +82,68 @@ class ImageSource(UserOwnedObject,Describable):
     """
     imageCreationDateTimeBestSoFar=''
     mostUpToDateImage = None
-    for installedImage in self.getInstalledImages():
+    for installedImage in self.installedImages:
       thisImagesCreationDateTime = installedImage.getCreationDateTime()
       if thisImagesCreationDateTime > imageCreationDateTimeBestSoFar:
         mostUpToDateImage = installedImage
         imageCreationDateTimeBestSoFar = thisImagesCreationDateTime
     return mostUpToDateImage
 
-  def getInstalledImages(self):
+  @property
+  def installedImages(self):
     """
     Return the installed images which are based on this image.
     """
     installedImagesBasedOnThisImageSource = []
-    for _,installedImage in self.getUser().getInstalledImages().items():
-      if installedImage.getImageSourceName() == self.getName() and installedImage.getSourceRepoId() == self.getRepository().getName():
+    for _,installedImage in self.user.installedImages.items():
+      if installedImage.imageSourceName == self.name and installedImage.sourceRepoId == self.repo.name:
         installedImagesBasedOnThisImageSource.append(installedImage)
     return installedImagesBasedOnThisImageSource
 
+  @property
+  def installedImagesDict(self):
+    dict = OrderedDict()
+    for image in self.installedImages:
+      dict[image.imageId] = image.serializeToDict()
+    return dict
+
   def getPermissionsFilePath(self):
     relativePath = self.getRelativePermissionsFilePath()
-    return os.path.join(self.getRepository().getRepoPath(),relativePath)
+    return os.path.join(self.repo.repoPath,relativePath)
 
   def getRelativePermissionsFilePath(self):
     if self.__explicitConfig is not None:
       return self.__explicitConfig["permissions-file"]
     if "dependent" in self.getRelativeSourceDir():
-      raise Exception(str(self.__explicitConfig)+"\n"+self.getName()+str(self.getRepository().keys()))
+      raise Exception(str(self.__explicitConfig)+"\n"+self.name+str(self.repo.keys()))
     return os.path.join(self.getRelativeSourceDir(),"permissions.json")
 
-  def getPermissions(self):
+  @property
+  def permissions(self):
     if not self.__permissions:
-      permissionsString = self.getRepository().getFileStructure().read(self.getRelativePermissionsFilePath())
-      initialPermissions = subuserlib.permissions.load(permissionsString=permissionsString)
-      self.__permissions = subuserlib.classes.permissions.Permissions(self.getUser(),initialPermissions,writePath=self.getPermissionsFilePath())
+      permissionsString = self.repo.fileStructure.read(self.getRelativePermissionsFilePath())
+      initialPermissions = subuserlib.permissions.load(permissionsString=permissionsString,logger=self.user.registry)
+      self.__permissions = subuserlib.classes.permissions.Permissions(self.user,initialPermissions,writePath=self.getPermissionsFilePath())
     return self.__permissions
 
-  def describe(self):
+  def description(self,rst=False):
+    if rst:
+      description = self.getIdentifier().replace("@","\\@") + "\n"
+      description += "-"*len(self.getIdentifier()) + "\n\n"
+    else:
+      description = self.getIdentifier() + "\n"
+    description += "To install this image run:" + (":\n\n" if rst else "\n")
+    description += "   subuser subuser add <subuser-name> "+self.getIdentifier()+"\n\n"
+    return description
+
+  def describe(self,rst=False):
     """
     Describe this ImageSource including it's default permissions.
 
     Prints to standard output.
     """
-    subuserlib.print.printWithoutCrashing(self.getIdentifier())
-    self.getPermissions().describe()
+    subuserlib.print.printWithoutCrashing(self.description(rst=rst))
+    self.permissions.describe(rst=rst)
 
   def build(self,parent,useCache=False):
     imageFileType = self.getImageFileType()
@@ -137,27 +157,27 @@ class ImageSource(UserOwnedObject,Describable):
           dockerfileContents = "FROM " + parent + "\n"
         else:
           dockerfileContents += line + "\n"
-    imageId = self.getUser().getDockerDaemon().build(relativeBuildContextPath=self.getImageDir(),repositoryFileStructure=self.getRepository().getFileStructure(),rm=True,dockerfile=dockerfileContents,useCache=useCache)
+    imageId = self.user.dockerDaemon.build(relativeBuildContextPath=self.getImageDir(),repositoryFileStructure=self.repo.fileStructure,rm=True,dockerfile=dockerfileContents,useCache=useCache)
     subuserSetupDockerFile = ""
     subuserSetupDockerFile += "FROM "+imageId+"\n"
     subuserSetupDockerFile += "RUN mkdir -p /subuser ; echo "+str(uuid.uuid4())+" > /subuser/uuid\n" # This ensures that all images have unique Ids.  Even images that are otherwise the same.
-    return self.getUser().getDockerDaemon().build(dockerfile=subuserSetupDockerFile,tag=self.getDockerImageTag(),useCache=False)
+    return self.user.dockerDaemon.build(dockerfile=subuserSetupDockerFile,tag=self.getDockerImageTag(),useCache=False)
 
   def getImageFile(self):
     if self.__explicitConfig:
       return self.__explicitConfig["image-file"]
     dockerfilePath = os.path.join(self.getImageDir(),"Dockerfile")
-    if self.getRepository().getFileStructure().exists(dockerfilePath):
+    if self.repo.fileStructure.exists(dockerfilePath):
       return dockerfilePath
     subuserImagefilePath = os.path.join(self.getImageDir(),"SubuserImagefile")
-    if self.getRepository().getFileStructure().exists(subuserImagefilePath):
+    if self.repo.fileStructure.exists(subuserImagefilePath):
       return subuserImagefilePath
 
   def getImageFileType(self):
     return os.path.basename(self.getImageFile())
 
   def getImageFileContents(self):
-    return self.getRepository().getFileStructure().read(self.getImageFile())
+    return self.repo.fileStructure.read(self.getImageFile())
 
   def getDependency(self):
     """
@@ -173,14 +193,14 @@ class ImageSource(UserOwnedObject,Describable):
         try:
           import subuserlib.resolve
           imageURI = line.split(" ")[1]
-          return subuserlib.resolve.resolveImageSource(self.getUser(),imageURI,contextRepository=self.getRepository(),allowLocalRepositories=False) #TODO, ImageSource names with spaces or other funny characters...
+          return subuserlib.resolve.resolveImageSource(self.user,imageURI,contextRepository=self.repo,allowLocalRepositories=False) #TODO, ImageSource names with spaces or other funny characters...
         except IndexError:
           raise exceptions.ImageBuildException("Syntax error in SubuserImagefile one line "+str(lineNumber)+":\n"+line)
         except KeyError:
-          raise exceptions.ImageBuildException("Error in "+self.getName()+"'s SubuserImagefile on line "+str(lineNumber)+"\n Subuser image does not exist: \""+imageURI+"\"")
+          raise exceptions.ImageBuildException("Error in "+self.name+"'s SubuserImagefile on line "+str(lineNumber)+"\n Subuser image does not exist: \""+imageURI+"\"")
       lineNumber+=1
     return None
 
   def getHash(self):
     """ Return the hash of the ``image`` directory. """
-    return self.getRepository().getFileStructure().hash(self.getImageDir())
+    return self.repo.fileStructure.hash(self.getImageDir())
