@@ -7,6 +7,7 @@ Each user has a set of images that have been installed.
 #external imports
 import os
 import json
+from collections import OrderedDict
 #internal imports
 from subuserlib.classes.userOwnedObject import UserOwnedObject
 from subuserlib.classes.describable import Describable
@@ -14,49 +15,38 @@ import subuserlib.classes.docker.dockerDaemon as dockerDaemon
 
 class InstalledImage(UserOwnedObject,Describable):
   def __init__(self,user,imageId,imageSourceName,sourceRepoId,imageSourceHash):
-    self.__imageId = imageId
-    self.__imageSourceHash = imageSourceHash
-    self.__imageSourceName = imageSourceName
-    self.__sourceRepoId = sourceRepoId
+    self.imageId = imageId
+    self.imageSourceHash = imageSourceHash
+    self.imageSourceName = imageSourceName
+    self.sourceRepoId = sourceRepoId
     self.__alreadyCheckedForUpdates = None
     UserOwnedObject.__init__(self,user)
 
-  def getImageId(self):
-    return self.__imageId
-
-  def getSourceRepoId(self):
-    return self.__sourceRepoId
-
-  def getImageSourceName(self):
-    return self.__imageSourceName
-
-  def getImageSource(self):
-    return self.getUser().getRegistry().getRepositories()[self.getSourceRepoId()][self.getImageSourceName()]
-
-  def getImageSourceHash(self):
-    return self.__imageSourceHash
+  @property
+  def imageSource(self):
+    return self.user.registry.repositories[self.sourceRepoId][self.imageSourceName]
 
   def isDockerImageThere(self):
     """
     Does the Docker daemon have an image with this imageId?
     """
-    return not (self.getUser().getDockerDaemon().getImageProperties(self.getImageId()) == None)
+    return not (self.user.dockerDaemon.getImageProperties(self.imageId) == None)
 
   def removeCachedRuntimes(self):
     """
     Remove cached runtime environments.
     """
-    pathToImagesRuntimeCacheDir = os.path.join(self.getUser().getConfig()["runtime-cache"],self.getImageId())
+    pathToImagesRuntimeCacheDir = os.path.join(self.user.config["runtime-cache"],self.imageId)
     try:
       for permissionsSpecificCacheInfoFileName in os.listdir(pathToImagesRuntimeCacheDir):
         permissionsSpecificCacheInfoFilePath = os.path.join(pathToImagesRuntimeCacheDir,permissionsSpecificCacheInfoFileName)
         with open(permissionsSpecificCacheInfoFilePath,mode='r') as permissionsSpecificCacheInfoFileHandle:
-          permissionsSpecificCacheInfo = json.load(permissionsSpecificCacheInfoFileHandle)
+          permissionsSpecificCacheInfo = json.load(permissionsSpecificCacheInfoFileHandle, object_pairs_hook=OrderedDict)
           try:
             try:
               imageId = permissionsSpecificCacheInfo['run-ready-image-id']
-              self.getUser().getRegistry().log("Removing runtime cache image %s"%imageId)
-              self.getUser().getDockerDaemon().removeImage(imageId)
+              self.user.registry.log("Removing runtime cache image %s"%imageId)
+              self.user.dockerDaemon.removeImage(imageId)
             except dockerDaemon.ImageDoesNotExistsException:
               pass
             os.remove(permissionsSpecificCacheInfoFilePath)
@@ -70,19 +60,26 @@ class InstalledImage(UserOwnedObject,Describable):
     Remove the image from the Docker daemon's image store.
     """
     try:
-      self.getUser().getRegistry().log("Removing image %s"%self.getImageId())
-      self.getUser().getDockerDaemon().removeImage(self.getImageId())
-    except (dockerDaemon.ImageDoesNotExistsException,dockerDaemon.ContainerDependsOnImageException,dockerDaemon.ServerErrorException) as e:
-      self.getUser().getRegistry().log("Error removing image: "+self.getImageId()+"\n"+str(e))
+      self.user.registry.log("Removing image %s"%self.imageId)
+      self.user.dockerDaemon.removeImage(self.imageId)
+    except (dockerDaemon.ImageDoesNotExistsException,dockerDaemon.ServerErrorException) as e:
+      self.user.registry.log("Error removing image: "+self.imageId+"\n"+str(e))
 
   def describe(self):
-    print("Image Id: "+self.getImageId())
+    print("Image Id: "+self.imageId)
     try:
-      print("Image source: "+self.getImageSource().getIdentifier())
+      print("Image source: "+self.imageSource.getIdentifier())
     except KeyError:
       print("Image is broken, image source does not exist!")
     print("Last update time: "+self.getCreationDateTime())
 
+  def serializeToDict(self):
+    imageAttributes = OrderedDict()
+    imageAttributes["image-source-hash"] = self.imageSourceHash
+    imageAttributes["image-source"] = self.imageSourceName
+    imageAttributes["source-repo"] = self.sourceRepoId
+    return imageAttributes
+
   def checkForUpdates(self):
     """
     Check for updates using the image's built in check-for-updates script. This launches the script as root in a privilageless container. Returns True if the image needs to be updated.
@@ -90,9 +87,9 @@ class InstalledImage(UserOwnedObject,Describable):
     if self.__alreadyCheckedForUpdates:
       return False
     self.__alreadyCheckedForUpdates = True
-    self.getUser().getRegistry().log("Checking for updates to: " + self.getImageSource().getIdentifier())
-    if self.getUser().getDockerDaemon().execute(["run","--rm","--entrypoint","/usr/bin/test",self.getImageId(),"-e","/subuser/check-for-updates"]) == 0:
-      returnCode = self.getUser().getDockerDaemon().execute(["run","--rm","--entrypoint","/subuser/check-for-updates",self.getImageId()])
+    self.user.registry.log("Checking for updates to: " + self.imageSource.getIdentifier())
+    if self.user.dockerDaemon.execute(["run","--rm","--entrypoint","/usr/bin/test",self.imageId,"-e","/subuser/check-for-updates"]) == 0:
+      returnCode = self.user.dockerDaemon.execute(["run","--rm","--entrypoint","/subuser/check-for-updates",self.imageId])
       if returnCode == 0:
         return True
     return False
@@ -101,7 +98,7 @@ class InstalledImage(UserOwnedObject,Describable):
     """
     Return the creation date/time of the installed docker image. Or None if the image does not exist.
     """
-    imageProperties = self.getUser().getDockerDaemon().getImageProperties(self.getImageId())
+    imageProperties = self.user.dockerDaemon.getImageProperties(self.imageId)
     if not imageProperties is None:
       return imageProperties["Created"]
     else:
@@ -112,7 +109,7 @@ class InstalledImage(UserOwnedObject,Describable):
     Return the list(lineage) of id of Docker image layers which goes from a base image to this image including all of the image's ancestors in order of dependency.
     """
     def getLineageRecursive(imageId):
-      imageProperties = self.getUser().getDockerDaemon().getImageProperties(imageId)
+      imageProperties = self.user.dockerDaemon.getImageProperties(imageId)
       if imageProperties == None:
         return []
         #sys.exit("Failed to get properties of image "+imageId)
@@ -120,7 +117,7 @@ class InstalledImage(UserOwnedObject,Describable):
         return getLineageRecursive(imageProperties["Parent"]) + [imageId]
       else:
         return [imageId]
-    return getLineageRecursive(self.getImageId())
+    return getLineageRecursive(self.imageId)
 
   def getImageLineage(self):
     """
@@ -129,6 +126,6 @@ class InstalledImage(UserOwnedObject,Describable):
     lineage = []
     dockerImageLayers = self.getLineageLayers()
     for dockerImageLayer in dockerImageLayers:
-      if dockerImageLayer in self.getUser().getInstalledImages():
-        lineage.append(self.getUser().getInstalledImages()[dockerImageLayer])
+      if dockerImageLayer in self.user.installedImages:
+        lineage.append(self.user.installedImages[dockerImageLayer])
     return lineage
