@@ -25,9 +25,9 @@ def getRecursiveDirectoryContents(directory):
 
 class Runtime(UserOwnedObject):
   def __init__(self,user,subuser,environment,extraDockerFlags=None,entrypoint = None):
-    self.__subuser = subuser
-    self.__environment = environment
-    self.__backgroundSuppressOutput = True
+    self.subuser = subuser
+    self.env = environment
+    self.backgroundSuppressOutput = True
     self.__backgroundCollectStdout = False
     self.__backgroundCollectStderr = False
     self.__executionSpoolReader = None
@@ -36,46 +36,53 @@ class Runtime(UserOwnedObject):
     else:
       self.__extraFlags = extraDockerFlags
     if not entrypoint:
-      self.entrypoint = self.getSubuser().getPermissions()["executable"]
+      self.entrypoint = self.subuser.permissions["executable"]
     else:
       self.entrypoint = entrypoint
-    self.__background = False
+    self.background = False
     if not subuserlib.test.testing:
-      self.__hostname = binascii.b2a_hex(os.urandom(10)).decode()
+      self.hostname = binascii.b2a_hex(os.urandom(10)).decode()
     else:
-      self.__hostname = "<random-hostname>"
+      self.hostname = "<random-hostname>"
     UserOwnedObject.__init__(self,user)
 
-  def getSubuser(self):
-    return self.__subuser
-
   def getRunReadyImageId(self):
     try:
-      return self.getSubuser().getRunReadyImage().getId()
+      image = self.subuser.getRunReadyImage().id
+      if image is None:
+        raise KeyError()
+      else:
+        return image
     except KeyError:
       sys.exit("""No run ready image is prepaired for this subuser. Please run:
 
 $ subuser repair
 """)
 
-  def getEnvironment(self):
-    return self.__environment
-
   def getEnvvar(self,var):
     try:
-      return self.getEnvironment()[var]
+      return self.env[var]
     except KeyError:
       sys.exit("The env var %s is not set. Are you sure you are running as a normal user?" % var)
 
   def getSerialDevices(self):
     return [device for device in os.listdir("/dev/") if device.startswith("ttyS") or device.startswith("ttyUSB") or device.startswith("ttyACM")]
 
+  def getGraphicsCardDevices(self):
+    try:
+      return os.listdir("/dev/dri")
+    except FileNotFoundError as e:
+      if subuserlib.test.testing:
+        return ["card0","controlD64"]
+      else:
+        system.exit("No graphics card available. /dev/dri does not exist.")
+
   def getCidFile(self):
-    return "/tmp/subuser-"+self.getSubuser().getName()
+    return "/tmp/subuser-"+self.subuser.name
 
   def getBasicFlags(self):
     common = ["--rm"]
-    if self.getBackground():
+    if self.background:
       return common + ["--cidfile",self.getCidFile()]
     else:
       if sys.stdout.isatty() and sys.stdin.isatty():
@@ -92,11 +99,11 @@ $ subuser repair
     Generate the arguments required to pass on a given ENV var to the container from the host.
     """
     try:
-      return ["-e",envVar+"="+self.getEnvironment()[envVar]]
+      return ["-e",envVar+"="+self.env[envVar]]
     except KeyError:
       return []
 
-  def getSoundArgs(self):
+  def getSoundCardArgs(self):
     soundArgs = []
     if os.path.exists("/dev/snd"):
       soundArgs += ["--volume=/dev/snd:/dev/snd:ro"]
@@ -109,36 +116,70 @@ $ subuser repair
         soundArgs += ["--device=/dev/dsp"]
     return soundArgs
 
+  def getPulseAudioArgs(self):
+    #TODO suport the plethora of alternate locations for these things...
+    if "PULSE_SERVER" in self.env:
+      pulseSocket = self.env["PULSE_SERVER"]
+    else:
+      pulseSocket = os.path.join("/run","user",str(self.user.endUser.uid),"pulse","native")
+    if "PULSE_COOKIE" in self.env:
+      pulseCookieFile = self.env["PULSE_COOKIE"]
+    else:
+      pulseCookieFile = os.path.join(self.subuser.homeDirOnHost,".pulse","cookie")
+    if os.path.exists(pulseSocket) and os.path.exists(pulseCookieFile):
+      return ["--volume="+pulseCookieFile+":/subuser/pulse/cookie"
+             ,"--volume="+pulseSocket+":/subuser/pulse/socket"
+             ,"-e"
+             ,"PULSE_COOKIE=/subuser/pulse/cookie"
+             ,"-e"
+             ,"PULSE_SERVER=/subuser/pulse/socket"]
+    else:
+      return []
+
+  def getBasicCommonPermissionFlags(self,bcps):
+    bcpd = collections.OrderedDict([
+      ("stateful-home", lambda p : ["--volume="+self.subuser.homeDirOnHost+":"+self.subuser.dockersideHome+":rw","-e","HOME="+self.subuser.dockersideHome] if p else ["-e","HOME="+self.subuser.dockersideHome]),
+      ("inherit-locale", lambda p : self.passOnEnvVar("LANG")+self.passOnEnvVar("LANGUAGE") if p else []),
+      ("inherit-timezone", lambda p : self.passOnEnvVar("TZ")+["--volume=/etc/localtime:/etc/localtime:ro"] if p else [])])
+    flags = []
+    for permission,flagGenerator in bcpd.items():
+      if type(bcps) == collections.OrderedDict and permission in bcps:
+        flags.extend(flagGenerator(bcps[permission]))
+      else:
+        flags.extend(flagGenerator(False))
+    return flags
+
   def getPermissionFlagDict(self):
     """
     This is a dictionary mapping permissions to functions which when given the permission's values return docker run flags.
     """
     return collections.OrderedDict([
      # Conservative permissions
-     ("stateful-home", lambda p : ["--volume="+self.getSubuser().getHomeDirOnHost()+":"+self.getSubuser().getDockersideHome()+":rw","-e","HOME="+self.getSubuser().getDockersideHome()] if p else ["-e","HOME="+self.getSubuser().getDockersideHome()]),
-     ("inherit-locale", lambda p : self.passOnEnvVar("LANG")+self.passOnEnvVar("LANGUAGE") if p else []),
-     ("inherit-timezone", lambda p : self.passOnEnvVar("TZ")+["--volume=/etc/localtime:/etc/localtime:ro"] if p else []),
+     ("basic-common-permissions", self.getBasicCommonPermissionFlags),
+     ("memory-limit", lambda p: ["--memory="+str(p)] if p else []),
+     ("max-cpus", lambda p: ["--cpus="+str(p)] if p else []),
      # Moderate permissions
-     ("gui", lambda p : ["-e","DISPLAY=unix:100","--volume",self.getSubuser().getX11Bridge().getServerSideX11Path()+":/tmp/.X11-unix:rw"] if p else []),
-     ("user-dirs", lambda userDirs : ["--volume="+os.path.join(self.getSubuser().getUser().getEndUser().homeDir,userDir)+":"+os.path.join("/subuser/userdirs/",userDir)+":rw" for userDir in userDirs]),
+     ("gui", lambda p : ["-e","DISPLAY=unix:100","--volume",self.subuser.x11Bridge.getServerSideX11Path()+":/tmp/.X11-unix:rw"] if p else []),
+     ("user-dirs", lambda userDirs : ["--volume="+os.path.join(self.subuser.user.endUser.homeDir,userDir)+":"+os.path.join("/subuser/userdirs/",userDir)+":rw" for userDir in userDirs]),
      ("inherit-envvars", lambda envVars: [arg for var in envVars for arg in self.passOnEnvVar (var)]),
-     ("sound-card", lambda p: self.getSoundArgs() if p else []),
+     ("sound-card", lambda p: self.getSoundCardArgs() if p else []),
+     ("pulseaudio", lambda p: self.getPulseAudioArgs() if p else []),
      ("webcam", lambda p: ["--device=/dev/"+device for device in os.listdir("/dev/") if device.startswith("video")] if p else []),
-     ("access-working-directory", lambda p: ["--volume="+os.getcwd()+":/pwd:rw","--workdir=/pwd"] if p else ["--workdir="+self.getSubuser().getDockersideHome()]),
+     ("access-working-directory", lambda p: ["--volume="+os.getcwd()+":/pwd:rw","--workdir=/pwd"] if p else ["--workdir="+self.subuser.dockersideHome]),
      ("allow-network-access", lambda p: ["--net=bridge"] if p else ["--net=none"]),
      # Liberal permissions
      ("x11", lambda p: ["-e","DISPLAY=unix"+self.getEnvvar('DISPLAY'),"--volume=/tmp/.X11-unix:/tmp/.X11-unix:rw","--volume="+self.getXautorityFilePath()+":/subuser/.Xauthority:ro","-e","XAUTHORITY=/subuser/.Xauthority"] if p else []),
      ("system-dirs", lambda systemDirs : ["--volume="+source+":"+dest+":rw" for source,dest in systemDirs.items()]),
-     ("graphics-card", lambda p: ["--device=/dev/dri/"+device for device in os.listdir("/dev/dri")] + ["--volume=/dev/dri/:/dev/dri/:ro"] if p else []),
+     ("graphics-card", lambda p: ["--device=/dev/dri/"+device for device in self.getGraphicsCardDevices()] + ["--volume=/dev/dri/:/dev/dri/:ro"] if p else []),
      ("serial-devices", lambda sd: ["--device=/dev/"+device for device in self.getSerialDevices()] if sd else []),
      ("system-dbus", lambda dbus: ["--volume=/var/run/dbus/system_bus_socket:/var/run/dbus/system_bus_socket:rw"] if dbus else []),
-     ("as-root", lambda root: ["--user=0"] if root else ["-e","USER="+self.getUser().getEndUser().name,"--user="+str(self.getUser().getEndUser().uid)]),
+     ("as-root", lambda root: ["--user=0"] if root else ["-e","USER=subuser","--user="+str(self.user.endUser.uid)]),
      # Anarchistic permissions
      ("run-commands-on-host", lambda p : ["--volume",self.getExecutionSpoolDir()+":/subuser/execute:rw"] if p else []),
      ("privileged", lambda p: ["--privileged"] if p else [])])
 
   def getExecutionSpoolDir(self):
-    return os.path.join(self.getUser().getConfig()["volumes-dir"],"execute",str(os.getpid()))
+    return os.path.join(self.user.config["volumes-dir"],"execute",str(os.getpid()))
 
   def getExecutionSpool(self):
     return os.path.join(self.getExecutionSpoolDir(),"spool")
@@ -149,14 +190,14 @@ $ subuser repair
     except (OSError,IOError):
       pass
     try:
-      self.getUser().getEndUser().makedirs(os.path.join(self.getExecutionSpoolDir()))
+      self.user.endUser.makedirs(os.path.join(self.getExecutionSpoolDir()))
     except (OSError,IOError):
       pass
     os.mkfifo(self.getExecutionSpool())
     executionSpoolReader = os.path.join(getSubuserDir(),"logic","execute-json-from-fifo")
     if not os.path.exists(executionSpoolReader):
       executionSpoolReader = subuserlib.executablePath.which("execute-json-from-fifo")
-    self.__executionSpoolReader = self.getUser().getEndUser().Popen([executionSpoolReader,self.getExecutionSpool()],cwd=self.getExecutionSpoolDir())
+    self.__executionSpoolReader = self.user.endUser.Popen([executionSpoolReader,self.getExecutionSpool()],cwd=self.getExecutionSpoolDir())
 
   def tearDownExecutionSpool(self):
     self.__executionSpoolReader.terminate()
@@ -166,15 +207,9 @@ $ subuser repair
     self.__extraFlags.append("-e")
     self.__extraFlags.append(envVar+"="+value)
 
-  def setHostname(self,hostname):
-    self.__hostname = hostname
-
-  def getHostname(self):
-    return self.__hostname
-
   def getHostnameFlag(self):
-    if not self.__hostname is None:
-      return ["--hostname",self.__hostname]
+    if not self.hostname is None:
+      return ["--hostname",self.hostname]
     else:
       return []
 
@@ -185,7 +220,7 @@ $ subuser repair
     flags = self.getBasicFlags()
     flags.extend(self.__extraFlags)
     permissionFlagDict = self.getPermissionFlagDict()
-    permissions = self.getSubuser().getPermissions()
+    permissions = self.subuser.permissions
     for permission, flagGenerator in permissionFlagDict.items():
       flags.extend(flagGenerator(permissions[permission]))
     flags.extend(self.getHostnameFlag())
@@ -198,41 +233,29 @@ $ subuser repair
     command = self.getCommand(args)
     return "docker '"+"' '".join(command)+"'"
 
-  def getBackground(self):
-    return self.__background
-
-  def setBackground(self,background):
-    self.__background = background
-
-  def getBackgroundSuppressOutput(self):
-    return self.__backgroundSuppressOutput
-
   def getBackgroundCollectOutput(self):
     return (self.__backgroundCollectStdout, self.__backgroundCollectStderr)
 
-  def setBackgroundSuppressOutput(self,suppressOutput):
-    self.__backgroundSuppressOutput = suppressOutput
-
   def setBackgroundCollectOutput(self,collectStdout,collectStderr):
     self.__backgroundCollectStdout = collectStdout
     self.__backgroundCollectStderr = collectStderr
 
   def getXautorityDirPath(self):
-    return os.path.join(self.getUser().getConfig()["volumes-dir"],"x11",str(os.getpid()),self.getSubuser().getName())
+    return os.path.join(self.user.config["volumes-dir"],"x11",str(os.getpid()),self.subuser.name)
 
   def getXautorityFilePath(self):
     return os.path.join(self.getXautorityDirPath(),".Xauthority")
 
   def setupXauth(self):
     try:
-      self.getUser().getEndUser().makedirs(self.getXautorityDirPath())
+      self.user.endUser.makedirs(self.getXautorityDirPath())
     except OSError: #Already exists
       pass
     try:
       os.remove(self.getXautorityFilePath())
     except OSError:
       pass
-    self.getUser().getEndUser().call(["xauth","extract",".Xauthority",self.getEnvvar("DISPLAY")],cwd=self.getXautorityDirPath())
+    self.user.endUser.call(["xauth","extract",".Xauthority",self.getEnvvar("DISPLAY")],cwd=self.getXautorityDirPath())
     with open(self.getXautorityFilePath(),"rb") as xauthFile:
       # The extracted Xauthority file has the following format(bytewise):
       # 1 0 0 [len(hostname)] [hostname-in-ascii] 0 1 [display-number-in-ascii] 0 22 ["MIT-MAGIC-COOKIE-1"-in-ascii] 0 20 [Magic number]
@@ -245,7 +268,7 @@ $ subuser repair
       rest = xauthFile.read()
     with open(self.getXautorityFilePath(),"wb") as xauthFile:
       xauthFile.write(start)
-      hostname = self.getHostname()
+      hostname = self.hostname
       hostname = hostname.encode("ascii")
       xauthFile.write(struct.pack("b",len(hostname)))
       xauthFile.write(hostname)
@@ -260,50 +283,50 @@ $ subuser repair
     def reallyRun():
       if not self.entrypoint:
         sys.exit("Cannot run subuser, no executable configured in permissions.json file.")
-      if self.getSubuser().getPermissions()["stateful-home"]:
-        self.getUser().getRegistry().log("Setting up subuser home dir.",verbosityLevel=4)
-        self.getSubuser().setupHomeDir()
-      if self.getSubuser().getPermissions()["stateful-home"] and self.getSubuser().getPermissions()["user-dirs"]:
-        self.getUser().getRegistry().log("Creating user dir symlinks in subuser home dir.",verbosityLevel=4)
-        userDirsDir = os.path.join(self.getSubuser().getHomeDirOnHost(),"Userdirs")
-        if os.path.islink(userDirsDir):
-          sys.exit("Please remove the old Userdirs directory, it is no longer needed. The path is:"+userDirsDir)
-      if self.getSubuser().getPermissions()["x11"]:
-        self.getUser().getRegistry().log("Generating xauth file.",verbosityLevel=4)
+      if self.subuser.permissions["basic-common-permissions"] and self.subuser.permissions["basic-common-permissions"]["stateful-home"]:
+        self.user.registry.log("Setting up subuser home dir.",verbosityLevel=4)
+        self.subuser.setupHomeDir()
+        if self.subuser.permissions["user-dirs"]:
+          self.user.registry.log("Creating user dir symlinks in subuser home dir.",verbosityLevel=4)
+          userDirsDir = os.path.join(self.subuser.homeDirOnHost,"Userdirs")
+          if os.path.islink(userDirsDir):
+            sys.exit("Please remove the old Userdirs directory, it is no longer needed. The path is:"+userDirsDir)
+      if self.subuser.permissions["x11"]:
+        self.user.registry.log("Generating xauth file.",verbosityLevel=4)
         self.setupXauth()
-      if self.getSubuser().getPermissions()["run-commands-on-host"]:
-        self.getUser().getRegistry().log("Launching execution spool daemon.",verbosityLevel=4)
+      if self.subuser.permissions["run-commands-on-host"]:
+        self.user.registry.log("Launching execution spool daemon.",verbosityLevel=4)
         self.setupExecutionSpool()
-      if self.getBackground():
+      if self.background:
         try:
           os.remove(self.getCidFile())
         except OSError:
           pass
       #Note, subusers with gui permission cannot be run in the background.
       # Make sure that everything is setup and ready to go.
-      if not self.getSubuser().getPermissions()["gui"] is None:
-        self.getUser().getRegistry().log("Requesting connection to X11 bridge.",verbosityLevel=4)
-        self.getSubuser().getX11Bridge().addClient()
-      self.getUser().getRegistry().log("Building run command.",verbosityLevel=4)
+      if self.subuser.permissions["gui"]:
+        self.user.registry.log("Requesting connection to X11 bridge.",verbosityLevel=4)
+        self.subuser.x11Bridge.addClient()
+      self.user.registry.log("Building run command.",verbosityLevel=4)
       command = self.getCommand(args)
       (collectStdout,collectStderr) = self.getBackgroundCollectOutput()
-      self.getUser().getRegistry().log("Running subuser with Docker.",verbosityLevel=4)
-      self.getUser().getRegistry().log(self.getPrettyCommand(args),verbosityLevel=4)
-      returnValue = self.getUser().getDockerDaemon().execute(command,background=self.getBackground(),backgroundSuppressOutput=self.getBackgroundSuppressOutput(),backgroundCollectStdout=collectStdout,backgroundCollectStderr=collectStderr)
-      if self.getSubuser().getPermissions()["run-commands-on-host"]:
-        self.getUser().getRegistry().log("Stopping execution spool.",verbosityLevel=4)
+      self.user.registry.log("Running subuser with Docker.",verbosityLevel=4)
+      self.user.registry.log(self.getPrettyCommand(args),verbosityLevel=4)
+      returnValue = self.user.dockerDaemon.execute(command,background=self.background,backgroundSuppressOutput=self.backgroundSuppressOutput,backgroundCollectStdout=collectStdout,backgroundCollectStderr=collectStderr)
+      if self.subuser.permissions["run-commands-on-host"]:
+        self.user.registry.log("Stopping execution spool.",verbosityLevel=4)
         self.tearDownExecutionSpool()
-      if not self.getSubuser().getPermissions()["gui"] is None:
-        self.getUser().getRegistry().log("Disconnecting from X11 bridge.",verbosityLevel=4)
-        self.getSubuser().getX11Bridge().removeClient()
-      if self.getBackground():
-        self.getUser().getRegistry().log("Waiting for CID file to be generated.",verbosityLevel=4)
+      if self.subuser.permissions["gui"]:
+        self.user.registry.log("Disconnecting from X11 bridge.",verbosityLevel=4)
+        self.subuser.x11Bridge.removeClient()
+      if self.background:
+        self.user.registry.log("Waiting for CID file to be generated.",verbosityLevel=4)
         while not os.path.exists(self.getCidFile()) or os.path.getsize(self.getCidFile()) == 0:
           time.sleep(0.05)
         with open(self.getCidFile(),"r") as cidFile:
-          self.getUser().getRegistry().log("Reading CID file.",verbosityLevel=4)
+          self.user.registry.log("Reading CID file.",verbosityLevel=4)
           containerId = cidFile.read()
-          container = self.getUser().getDockerDaemon().getContainer(containerId)
+          container = self.user.dockerDaemon.getContainer(containerId)
           if container is None:
             sys.exit("Container failed to start:"+containerId)
           os.remove(self.getCidFile())
