@@ -47,17 +47,6 @@ For installation instructions see <https://www.docker.io/gettingstarted/#h_insta
 
 # docker -d
 """)
-
-  username = getpass.getuser()
-  if not os.getuid() == 0:
-    try:
-      groupmembers = grp.getgrnam("docker").gr_mem
-    except KeyError:
-      groupmembers = grp.getgrnam("dockerroot").gr_mem
-    if not username in groupmembers:
-      sys.exit("""Error: You are not a member of the docker group.
-
-To learn how to become a member of the docker group please watch this video: <http://www.youtube.com/watch?v=ahgRx5U4V7E>""")
   verified = True
   return executable
 
@@ -72,3 +61,12 @@ def runBackground(args,cwd=None,suppressOutput=True,collectStdout=False,collectS
   Run docker with the given command line arguments. Return Docker's pid.
   """
   return subprocessExtras.callBackground([getAndVerifyExecutable()]+args,cwd,suppressOutput=suppressOutput,collectStdout=collectStdout,collectStderr=collectStderr)
+
+def buildImageTag(tag,hash):
+  tag ='{:-<95.95}{:-<32.32}'.format(tag,hash)
+  def isvalid(c):
+    if c in "-_.":
+      return True
+    return c.isalpha() or c.isdigit()
+  return ''.join([c if isvalid(c) else "-" for c in tag])
+
